@@ -24,6 +24,7 @@
 #include <string.h>
 #include <gda-attributes-manager.h>
 #include <gda-value.h>
+#include <gda-mutex.h>
 
 /*
  * Structure for the attribute names
@@ -49,6 +50,7 @@ static void objattrs_unref (ObjAttrs *attrs);
 
 
 struct _GdaAttributesManager {
+	GdaMutex                   *mutex;
 	gboolean                    for_objects; /* TRUE if key->data are GObjects */
 	GdaAttributesManagerSignal  signal_func;
 	gpointer                    signal_data;
@@ -105,6 +107,7 @@ gda_attributes_manager_new (gboolean for_objects, GdaAttributesManagerSignal sig
 	GdaAttributesManager *mgr;
 
 	mgr = g_new0 (GdaAttributesManager, 1);
+	mgr->mutex = gda_mutex_new  ();
 	mgr->obj_hash = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL,
 					       (GDestroyNotify) objattrs_unref);
 	mgr->for_objects = for_objects;
@@ -115,8 +118,26 @@ gda_attributes_manager_new (gboolean for_objects, GdaAttributesManagerSignal sig
 }
 
 static void
-foreach_destroy_func (gpointer ptr, ObjAttrs *attrs, gpointer data)
+obj_destroyed_cb (ObjAttrs *attrs, GObject *where_the_object_was)
 {
+	GdaMutex *mutex;
+
+	/* rem: we need to keep a pointer to mutex because attrs may be destroyed
+	 * in g_hash_table_remove */
+	mutex = attrs->mgr->mutex;
+
+	gda_mutex_lock (mutex);
+	attrs->objects = g_slist_remove (attrs->objects, where_the_object_was);
+	g_hash_table_remove (attrs->mgr->obj_hash, where_the_object_was);
+	gda_mutex_unlock (mutex);
+}
+
+static void
+foreach_destroy_func (gpointer ptr, ObjAttrs *attrs, GdaAttributesManager *mgr)
+{
+	if (mgr->for_objects)
+		g_object_weak_unref (G_OBJECT (ptr), 
+				     (GWeakNotify) obj_destroyed_cb, attrs);
 	attrs->objects = g_slist_remove (attrs->objects, ptr);
 }
 
@@ -129,16 +150,14 @@ foreach_destroy_func (gpointer ptr, ObjAttrs *attrs, gpointer data)
 void
 gda_attributes_manager_free (GdaAttributesManager *mgr)
 {
-	g_hash_table_foreach (mgr->obj_hash, (GHFunc) foreach_destroy_func, NULL);
+	GdaMutex *mutex;
+	mutex = mgr->mutex;
+	gda_mutex_lock (mutex);
+	g_hash_table_foreach (mgr->obj_hash, (GHFunc) foreach_destroy_func, mgr);
 	g_hash_table_destroy (mgr->obj_hash);
 	g_free (mgr);
-}
-
-static void
-obj_destroyed_cb (ObjAttrs *attrs, GObject *where_the_object_was)
-{
-	attrs->objects = g_slist_remove (attrs->objects, where_the_object_was);
-	g_hash_table_remove (attrs->mgr->obj_hash, where_the_object_was);
+	gda_mutex_unlock (mutex);
+	gda_mutex_free (mutex);
 }
 
 typedef struct {
@@ -158,6 +177,8 @@ manager_real_set (GdaAttributesManager *mgr, gpointer ptr,
 	g_return_if_fail (att_name);
 	if (mgr->for_objects) 
 		g_return_if_fail (G_IS_OBJECT (ptr));
+	
+	gda_mutex_lock (mgr->mutex);
 
 	/* pick up the correct ObjAttrs */
 	objattrs = g_hash_table_lookup (mgr->obj_hash, ptr);
@@ -220,6 +241,8 @@ manager_real_set (GdaAttributesManager *mgr, gpointer ptr,
 	}
 	if (mgr->signal_func && mgr->for_objects)
 		mgr->signal_func ((GObject*) ptr, att_name, value, mgr->signal_data);
+
+	gda_mutex_unlock (mgr->mutex);
 }
 
 /**
@@ -277,14 +300,19 @@ const GValue *
 gda_attributes_manager_get (GdaAttributesManager *mgr, gpointer ptr, const gchar *att_name)
 {
 	ObjAttrs *objattrs;
+	const GValue *cvalue = NULL;
+
+	gda_mutex_lock (mgr->mutex);
 
 	objattrs = g_hash_table_lookup (mgr->obj_hash, ptr);
 	if (objattrs) {
 		AttName attname;
 		attname.att_name = (gchar*) att_name;
-		return g_hash_table_lookup (objattrs->values_hash, &attname);
+		cvalue = g_hash_table_lookup (objattrs->values_hash, &attname);
 	}
-	return NULL;
+
+	gda_mutex_unlock (mgr->mutex);
+	return cvalue;
 }
 
 /**
@@ -302,9 +330,17 @@ gda_attributes_manager_copy (GdaAttributesManager *from_mgr, gpointer *from,
 			     GdaAttributesManager *to_mgr, gpointer *to)
 {
 	ObjAttrs *from_objattrs, *to_objattrs;
+
+	gda_mutex_lock (from_mgr->mutex);
+	gda_mutex_lock (to_mgr->mutex);
+
 	from_objattrs = g_hash_table_lookup (from_mgr->obj_hash, from);
-	if (!from_objattrs)
+	if (!from_objattrs) {
+		gda_mutex_unlock (from_mgr->mutex);
+		gda_mutex_unlock (to_mgr->mutex);
 		return;
+	}
+
 	to_objattrs = g_hash_table_lookup (to_mgr->obj_hash, to);
 
 	if ((from_mgr == to_mgr) && !to_objattrs) {
@@ -313,6 +349,9 @@ gda_attributes_manager_copy (GdaAttributesManager *from_mgr, gpointer *from,
 
 		if (from_mgr->for_objects) 
 			g_object_weak_ref (G_OBJECT (to), (GWeakNotify) obj_destroyed_cb, from_objattrs);
+
+		gda_mutex_unlock (from_mgr->mutex);
+		gda_mutex_unlock (to_mgr->mutex);
 		return;
 	}
 
@@ -321,6 +360,9 @@ gda_attributes_manager_copy (GdaAttributesManager *from_mgr, gpointer *from,
 	cdata.to_mgr = to_mgr;
 	cdata.ptr = to;
 	g_hash_table_foreach (from_objattrs->values_hash, (GHFunc) foreach_copy_func, &cdata);
+
+	gda_mutex_unlock (from_mgr->mutex);
+	gda_mutex_unlock (to_mgr->mutex);
 }
 
 static void
@@ -343,11 +385,16 @@ void
 gda_attributes_manager_clear (GdaAttributesManager *mgr, gpointer ptr)
 {
 	ObjAttrs *objattrs;
+
+	gda_mutex_lock (mgr->mutex);
+
 	objattrs = g_hash_table_lookup (mgr->obj_hash, ptr);
 	if (objattrs) {
 		objattrs->objects = g_slist_remove (objattrs->objects, ptr);
 		g_hash_table_remove (mgr->obj_hash, ptr);
 	}
+
+	gda_mutex_unlock (mgr->mutex);
 }
 
 typedef struct {
@@ -374,6 +421,8 @@ gda_attributes_manager_foreach (GdaAttributesManager *mgr, gpointer ptr,
 	g_return_if_fail (func);
 	g_return_if_fail (ptr);
 
+	gda_mutex_lock (mgr->mutex);
+
 	objattrs = g_hash_table_lookup (mgr->obj_hash, ptr);
 	if (objattrs) {
 		FData fdata;
@@ -382,6 +431,8 @@ gda_attributes_manager_foreach (GdaAttributesManager *mgr, gpointer ptr,
 		fdata.data = data;
 		g_hash_table_foreach (objattrs->values_hash, (GHFunc) foreach_foreach_func, &fdata);
 	}
+
+	gda_mutex_unlock (mgr->mutex);
 }
 
 static void
