@@ -826,6 +826,8 @@ fill_tables_views_model (GdaConnection *cnc,
                                             FALSE, NULL)) /* table_owner */
                                 retval = FALSE;
                 }
+		else
+			gda_value_free (ncvalue);
         }
         g_object_unref (tmpmodel);
 
@@ -2240,6 +2242,42 @@ _gda_sqlite_meta_routine_par (GdaServerProvider *prov, GdaConnection *cnc,
 	return TRUE;
 }
 
+gboolean
+_gda_sqlite_meta__indexes_tab (GdaServerProvider *prov, GdaConnection *cnc, 
+			       GdaMetaStore *store, GdaMetaContext *context, GError **error)
+{
+	//TO_IMPLEMENT;
+	return TRUE;
+}
+
+gboolean
+_gda_sqlite_meta_indexes_tab (GdaServerProvider *prov, GdaConnection *cnc, 
+			      GdaMetaStore *store, GdaMetaContext *context, GError **error,
+			      const GValue *table_catalog, const GValue *table_schema, const GValue *table_name,
+			      const GValue *index_name_n)
+{
+	//TO_IMPLEMENT;
+	return TRUE;
+}
+
+gboolean
+_gda_sqlite_meta__index_cols (GdaServerProvider *prov, GdaConnection *cnc, 
+			      GdaMetaStore *store, GdaMetaContext *context, GError **error)
+{
+	//TO_IMPLEMENT;
+	return TRUE;
+}
+
+gboolean
+_gda_sqlite_meta_index_cols (GdaServerProvider *prov, GdaConnection *cnc, 
+			     GdaMetaStore *store, GdaMetaContext *context, GError **error,
+			     const GValue *table_catalog, const GValue *table_schema,
+			     const GValue *table_name, const GValue *index_name)
+{
+	//TO_IMPLEMENT;
+	return TRUE;
+}
+
 /*
  * @...: a list of TRUE/FALSE, GValue*  -- if TRUE then the following GValue must be freed
  */
